@@ -149,7 +149,7 @@ class SQLObject:
     def delete(self, search: str, keys: list=USUAL_KEYS, exact: bool=False):
         search = re.escape(search)
         if exact:
-            not_match = lambda item: not re.search(fr'\w*[.]*{search}$', item)
+            not_match = lambda item: not re.search(fr'([^\w+]|[^_]){search}$', item)
         else:
             not_match = lambda item: search not in item
         for key in keys:
@@ -263,6 +263,7 @@ class Function(Code):
     append_param = False
 
     def __init__(self, *params: list):
+        # ----------------------------------------
         def set_func_types(param):
             if self.auto_convert and isinstance(param, Function):
                 func = param
@@ -275,6 +276,7 @@ class Function(Code):
                 if unfriendly:
                     return Cast(func, main_param)
             return param
+        # ----------------------------------------
         self.params = [set_func_types(p) for p in params]
         self.pattern = self.get_pattern()
         super().__init__()
@@ -300,34 +302,72 @@ class Function(Code):
     def help(cls) -> str:
         descr = ' '.join(B.__name__ for B in cls.__bases__)
         params = cls.inputs or ''
+        docstring = cls.__doc__ if cls.__doc__ else ''
         return cls().get_pattern().format(
             func_name=f'{descr} {cls.__name__}',
             params=cls.separator.join(str(p) for p in params)
-        ) + f'  Return {cls.output}'
+        ) + f'  Return {cls.output}' + docstring
 
-    def set_main_param(self, name: str, main: SQLObject) -> bool:
-        nested_functions = [
-            param for param in self.params if isinstance(param, Function)
-        ]
-        for func in nested_functions:
-            if func.inputs:
-                func.set_main_param(name, main)
-                return
+    def set_main_param(self, name: str, main: SQLObject, root: 'Function'=None) -> bool:
+        map = {
+            Function: [],
+            str: []
+        }
+        for i, param in enumerate(self.params):
+            for ptype in map:
+                if isinstance(param, ptype):
+                    map[ptype].append(i)
+        if not root:
+            root = self
+        for i in map[Function]:
+            func = self.params[i]
+            if not func.inputs:
+                continue
+            if func.set_main_param(name, main, root):
+                return True
+        if 0 < len(self.params) >= len(self.inputs) and Ellipsis not in self.inputs:
+            root.As(name)
+            if not map[str]:
+                return False
+            i = map[str][0]
+            name = self.params.pop(i)
         new_params = [Field.format(name, main)]
         if self.append_param:
             self.params += new_params
         else:
-            self.params = new_params + self.params         
+            self.params = new_params + self.params
+        return True
 
     def format(self, name: str, main: SQLObject) -> str:
+        if isinstance(self, Frame) and Partition.params:
+            self.over(**Partition.params)
+            Partition.params = None
         if name not in '*_':
             self.set_main_param(name, main)
         return str(self)
 
+    @classmethod
+    def list_all(cls, function: callable = print):
+        LINE_SEPARATOR = '-'*20
+        children = [
+            f.help() for f in Function.descendants()
+        ]
+        children.sort()
+        function(
+            "{}\n{}\n{}".format(
+                '='*20,
+                f"\n{LINE_SEPARATOR}\n".join(children),
+                '='*20,
+            )
+        )
+
 
 
 # ---- String Functions: ---------------------------------
 class SubString(Function):
+    """
+    Extracts a portion of a string
+    """
     inputs = [CHAR, INT, INT]
     output = CHAR
 
@@ -338,16 +378,25 @@ class SubString(Function):
 
 # ---- Numeric Functions: --------------------------------
 class Round(Function):
-    inputs = [FLOAT]
+    """
+    Rounds a number to a specified number of decimal places
+    """
+    inputs = [FLOAT, INT]
     output = FLOAT
 
 class Trunc(Function):
+    """
+    Truncate a number to a integer precision
+    """
     inputs = [FLOAT]
-    output = FLOAT
+    output = INT
 
 # --- Date Functions: ------------------------------------
 class DateDiff(Function):
-    inputs = [DATE]
+    """
+    Returns the difference between two dates
+    """
+    inputs = [DATE, DATE]
     output = DATE
     append_param = True
 
@@ -384,6 +433,11 @@ class DatePart(Function):
             return database_type[self.dialect]
         return super().get_pattern()
 
+    @classmethod
+    def help(cls):
+        result = super().help()
+        return result.replace('DatePart ', "Date Function ")
+
 class Year(DatePart):
     ...
 class Month(DatePart):
@@ -416,12 +470,7 @@ class Frame:
         """
         keywords = ''
         for field, obj in args.items():
-            is_valid = any([
-                obj is OrderBy, obj is DescOrderBy,
-                obj is Partition,
-                isinstance(obj, Rows),
-            ])
-            if not is_valid:
+            if not hasattr(obj, "cls_to_str"):
                 continue
             keywords += '{}{}'.format(
                 '\n\t\t' if self.break_lines else ' ',
@@ -442,36 +491,72 @@ class Window(Frame):
 
 # ---- Aggregate Functions: -------------------------------
 class Avg(Aggregate, Function):
+    """
+     Calculate the average (arithmetic mean) of a 
+    set of numeric values within a specified column
+    """
     ...
 class Min(Aggregate, Function):
+    """
+    Returns the smallest value of the selected column.
+    """
     ...
 class Max(Aggregate, Function):
+    """
+    Returns the largest value of the selected column.
+    """
     ...
 class Sum(Aggregate, Function):
     ...
 class Count(Aggregate, Function):
+    """
+    Return the number of rows that matches a specified criterion.
+    """
     ...
 
 # ---- Window Functions: -----------------------------------
 class Row_Number(Window, Function):
+    """
+    The sequential number of a row
+    within a partition of a result set.
+    """
     output = INT
 
 class Rank(Window, Function):
+    """
+     Assign a rank to each row within a result
+    set, based on a specified ordering of data.
+    """
     output = INT
 
 class Lag(Window, Function):
+    """
+     Allows for accessing data from a 
+    preceding row within the same result set.
+    """
     output = ANY
 
 class Lead(Window, Function):
+    """
+     Allows for accessing data from a 
+    subsequent row within the same result set.
+    """
     output = ANY
 
 
 # ---- Conversions and other Functions: ---------------------
 class Coalesce(Function):
+    """
+    Returns the first non-NULL expression
+    from a list of expressions.
+    """
     inputs = [ANY]
     output = ANY
     
 class Cast(Function):
+    """
+    Converts a value (of any type) into a specified datatype.
+    """
     inputs = [ANY]
     output = ANY
     separator = ' As '
@@ -508,7 +593,7 @@ class FieldList:
                 f.strip() for f in fields.split(self.separator)
             ]
         self.fields = fields
-        self.class_types = class_types
+        self.class_types = TO_LIST(class_types)
         self.ziped = ziped
 
     def add(self, name: str, main: SQLObject):
@@ -962,9 +1047,18 @@ class OrderBy(Clause):
         return f"{ORDER_BY} {field}{cls.sort.value}"
 
 class Partition:
+    params = None
+
     @classmethod
     def cls_to_str(cls, field: str) -> str:
         return f'PARTITION BY {field}'
+    
+    def __init__(self, content):
+        self.content = content
+
+    def add(self, name: str, main: SQLObject):
+        Partition.params = {name: Partition}  # <=== class variable!
+        self.content.add(name, main)
 
 
 class GroupBy(Clause):
@@ -1564,12 +1658,13 @@ class JoinType(Enum):
 
 class SQLParser(Parser):
     REGEX = {}
+    SUB_QUERIES_AS_CONDITIONS = True
 
     def prepare(self):
         keywords = '|'.join(k + r'\b' for k in KEYWORD)
         flags = re.IGNORECASE + re.MULTILINE
         self.REGEX['keywords'] = re.compile(f'({keywords})', flags)
-        self.REGEX['subquery'] = re.compile(r'(\w\.)*\w+ +in +\(SELECT.*?\)', flags)
+        self.REGEX['subquery'] = re.compile(r'(\w+[.])*\w+\s+in\s*[(]\s*SELECT\s+', flags)
 
     def eval(self, txt: str):
         def find_last_word(pos: int) -> int:
@@ -1584,22 +1679,27 @@ class SQLParser(Parser):
                     found.add(WORD)
                 elif txt[i] == '.':
                     found.remove(WORD)
-        def find_parenthesis(pos: int) -> int:
-            for i in range(pos, len(txt)-1):
-                if txt[i] == ')':
-                    return i+1
-        result = {}
+        def find_parenthesis(found) -> tuple:
+            start, end = found.span()
+            CHAR_VALUES = {
+                '(': +1,
+                ')': -1
+            }
+            remaining = 1
+            for i, char in enumerate(txt[end:]):
+                remaining += CHAR_VALUES.get(char, 0)
+                if char == ')' and not remaining:
+                    end += i
+                    break
+            return start, end
+        result, subqueries = {}, {}
         found = self.REGEX['subquery'].search(txt)
         while found:
-            start, end = found.span()
-            inner = txt[start: end]
-            if inner.count('(') > inner.count(')'):
-                end = find_parenthesis(end)
-                inner = txt[start: end-1]
-            fld, *inner = re.split(r' IN | in', inner, maxsplit=1)
+            start, end = find_parenthesis(found)
+            fld, _, *inner = re.split(r'\s+(in|IN)\s+', txt[start: end], maxsplit=1)
             if fld.upper() == 'NOT':
                 pos = find_last_word(start)
-                fld = txt[pos: start].strip() # [To-Do] Use the value of `fld`
+                fld = txt[pos: start].strip()
                 start = pos
                 target_class = NotSelectIN
             else:
@@ -1608,7 +1708,12 @@ class SQLParser(Parser):
                 ' '.join(re.sub(r'^\(', '', s.strip()) for s in inner),
                 class_type=target_class
             ).queries[0]
-            result[obj.alias] = obj
+            if self.SUB_QUERIES_AS_CONDITIONS:
+                *alias, fld = fld.split('.')
+                alias = '' if not alias else alias[0]
+                subqueries.setdefault(alias, {})[fld] = obj
+            else:
+                result[obj.alias] = obj
             txt = txt[:start-1] + txt[end+1:]
             found = self.REGEX['subquery'].search(txt)
         tokens = [t.strip() for t in self.REGEX['keywords'].split(txt) if t.strip()]
@@ -1634,6 +1739,8 @@ class SQLParser(Parser):
                         for fld in self.class_type.split_fields(values[key], key)
                         if (fld != '*' and len(tables) == 1) or obj.match(fld, key)
                     ]
+                if obj.alias in subqueries:
+                    obj.__call__(**subqueries[obj.alias])                    
                 result[obj.alias] = obj
         self.queries = list( result.values() )
 
@@ -1648,7 +1755,7 @@ class CypherParser(Parser):
 
     def prepare(self):
         self.REGEX['separator'] = re.compile(fr'({self.CHAR_SET}|->|<-|{self.KEYWORDS})')
-        self.REGEX['condition'] = re.compile(r'(^\w+)|([<>=])')
+        self.REGEX['condition'] = re.compile(r'(\w+)(\s*[<>=]+\s*["\']*[\d+|\w+|\+\-\*\/]*["\']*)')
         self.REGEX['alias_pos'] = re.compile(r'(\w+)[.](\w+)')
         self.join_type = JoinType.INNER
         self.TOKEN_METHODS = {
@@ -2527,32 +2634,21 @@ def detect(text: str, join_method = join_queries, format: str='') -> Select | li
     return result
 # ===========================================================================================//
 
-
-if __name__ == '__main__':
-    cte = CTEFactory(
-        txt='''
-            ClienteMaisRecente[
-            Pedidos(cliente_id@, ^max$dataPedido:ultimoPedido)
-            ]
-            PedidosMaisRecentes[
-                [1](cliente_id*) <- Pedidos(cliente_id)
-            ]
-
-            [-1](**)
-        '''
+if __name__ == "__main__":
+    TABLE_NAME = 'Sales s'
+    DATE_FIELD = 'ref_date'
+    print('▒'*50)
+    q1 = Select(
+        TABLE_NAME, 
+        ref_year=Year(DATE_FIELD),
+        price=Sum().As('total').over(ref_year=Partition)
     )
-    print(cte)
-    # --------------------------------------------------------------------------
-    # print(  CTEFactory(
-    #     txt='''
-    #         #Empregado #Cliente #Fornecedor
-    #         Todas_as_pessoas[[1]     [2]     [3]]
-    #         [-1](**, ano*) <- Meta(ano, qt_ideal)
-    #     ''',
-    #     template='''
-    #         Vendas_por_{t}[
-    #             Vendas(year$data:ano@, sum$quantidade:qt_vendida,
-    #             {f}) -> {t}(id, nome:nome_pessoa@)
-    #         ]
-    #     '''
-    # )  )
+    print(q1)
+    print('='*50)
+    q2 = Select(
+        TABLE_NAME, 
+        ref_year=Partition( Year(DATE_FIELD) ),
+        total=Sum('price'),
+    )
+    print(q2)
+    print('▒'*50)
